@@ -69,6 +69,14 @@ fn main() {
                                 recursive_namespaced_insert(name, field_value, values);
                             }
                         }
+                        TrackedDataValue::Enum { variant, fields } => {
+                            values.enum_variant.insert(namespaced_name.clone(), variant);
+                            for (field_name, field_value) in fields {
+                                let name = format!("{}:{}", namespaced_name, field_name);
+                                recursive_namespaced_insert(name, field_value, values);
+                            }
+                        }
+                        TrackedDataValue::EnumVariant(_) => {} // This will never be sent to us, so ignore it.
                         TrackedDataValue::Trigger => {}
                     }
                 }
@@ -96,14 +104,14 @@ fn main() {
                             .iter_mut()
                             .find(|var| var.name == namespaced_name[0])
                             .unwrap();
-                        
+
                         // Iterate into fields
                         for name in &namespaced_name[1..] {
                             value = match value.data_type {
-                                TrackedDataRepr::Struct { ref mut fields, .. } => fields
-                                    .iter_mut()
-                                    .find(|var| var.name == *name)
-                                    .unwrap(),
+                                TrackedDataRepr::Struct { ref mut fields, .. }
+                                | TrackedDataRepr::Enum { ref mut fields, .. } => {
+                                    fields.iter_mut().find(|var| var.name == *name).unwrap()
+                                }
                                 _ => panic!(),
                             };
                         }
@@ -122,7 +130,6 @@ fn main() {
                         );
                     }
                     Message::Quit => {
-                        eprintln!("Did quit");
                         quit = true;
                     }
                 }
@@ -189,7 +196,7 @@ fn main() {
             }
         }
         glutin::event::Event::WindowEvent { event, .. } => {
-            egui.on_event(event, control_flow);
+            egui.on_event(&event);
             display.gl_window().window().request_redraw();
         }
         _ => (),
@@ -203,6 +210,7 @@ struct Values {
     f64: HashMap<String, f64>,
     bool: HashMap<String, bool>,
     str: HashMap<String, String>,
+    enum_variant: HashMap<String, String>,
 }
 
 enum Message {
@@ -251,6 +259,59 @@ fn recursive_ui<'a>(
                         });
                 });
             }
+            TrackedDataRepr::Enum {
+                name,
+                variants,
+                fields,
+                triggers,
+            } => {
+                ui.collapsing(name, |ui| {
+                    egui::Grid::new(format!("{}_grid", &namespaced_name))
+                        .striped(true)
+                        .spacing([40.0, 4.0])
+                        .show(ui, |ui| {
+                            let selected_variant = values
+                                .enum_variant
+                                .entry(namespaced_name.clone())
+                                .or_default();
+                            let mut changed = false;
+                            egui::ComboBox::from_id_source(format!("{}_variant", &namespaced_name))
+                                .selected_text(selected_variant.clone())
+                                .show_ui(ui, |ui| {
+                                    for variant in variants {
+                                        changed |= ui
+                                            .selectable_value(
+                                                selected_variant,
+                                                variant.clone(),
+                                                variant,
+                                            )
+                                            .clicked();
+                                    }
+                                });
+                            if changed {
+                                modified_variables.push((
+                                    namespaced_name.clone(),
+                                    TrackedDataValue::EnumVariant(
+                                        values
+                                            .enum_variant
+                                            .entry(namespaced_name.clone())
+                                            .or_default()
+                                            .clone(),
+                                    ),
+                                ))
+                            }
+                            ui.end_row();
+                            recursive_ui(
+                                ui,
+                                values,
+                                modified_variables,
+                                fields.iter(),
+                                triggers.clone().into_iter(),
+                                namespaced_name,
+                            )
+                        });
+                });
+            }
             TrackedDataRepr::SignedSlider {
                 storage_min,
                 storage_max,
